@@ -1,5 +1,5 @@
 /* Copyright (c) 2011 Nanakos Chrysostomos <nanakos@wired-net.gr>
-   Simple and lightweight Yubikey OTP-OAUTH/HOTP Validation Server
+   Simple and lightweight Yubikey OTP-OATH/HOTP Validation Server
 
    yubiserver-admin is placed under the GNU General Public License, version 2 or later.
 
@@ -30,6 +30,7 @@
 #include <gcrypt.h>
 #include <sqlite3.h>
 #include <time.h>
+#include <sys/timeb.h>
 #include <getopt.h>
 #include <math.h>
 
@@ -38,12 +39,13 @@
 #include "yubiserver.h"
 
 int yubikey_table = 0;
-int oauth_table = 0;
+int oath_table = 0;
 int api_table = 0;
+int ext_db = 0;
 int Argc;
 char **Argv;
 #define YUBIKEY	0
-#define OAUTH	1
+#define OATH	1
 #define API	2
 
 static char encoding_table[] = {'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H',
@@ -86,7 +88,7 @@ void print_table(char *table,int id)
 	int retval,rows=0;
 	char *query = NULL;
 
-	if(id==YUBIKEY || id==OAUTH)
+	if(id==YUBIKEY || id==OATH)
 	{
 		query = (char *)malloc((size_t)100);
 		sprintf(query,"SELECT nickname,publicname,active FROM %s",table);
@@ -119,7 +121,7 @@ void print_table(char *table,int id)
 
 		if(retval==SQLITE_ROW)
 		{
-			if(id==YUBIKEY || id==OAUTH)
+			if(id==YUBIKEY || id==OATH)
 			{
 				if(!rows) printf("%-20s %-20s %-20s\n","[Username]","[Public Token ID]","Active");
 				printf("%-20s %-20s %-20s\n",sqlite3_column_text(stmt,0),sqlite3_column_text(stmt,1),sqlite3_column_text(stmt,2));
@@ -157,9 +159,11 @@ void db_ops(char *table,char *user,int operation,int id)
         char *query = NULL;
 	time_t t;
 	struct tm *tmp;
+	struct timeb tp;
 	char *cdatetime = NULL;
+	char *cdatetime_tmp = NULL;
 
-	if(id == YUBIKEY || id == OAUTH)
+	if(id == YUBIKEY || id == OATH)
 	{
 		if(operation == ENABLE_USER || operation == DISABLE_USER || operation == DELETE_USER)
 		{
@@ -268,16 +272,35 @@ void db_ops(char *table,char *user,int operation,int id)
 		{
 			if(id==YUBIKEY)
 			{
-				if(strlen(user) > 16 || strlen(Argv[4])!=12 || strlen(Argv[5]) != 12 || strlen(Argv[6]) != 32)
+				if(!ext_db)
 				{
-					fprintf(stderr,"Parameters are not correct. Please try again.\n");
-					fprintf(stderr,"Public token ID must be 12 characters long.\n");
-					fprintf(stderr,"Secret token ID must be 12 characters long.\n");
-					fprintf(stderr,"AES key must be 32 characters long.\n");
-					exit(2);
+					if((strlen(user) > 16 || strlen(Argv[4])!=12 || strlen(Argv[5]) != 12 || strlen(Argv[6]) != 32))
+					{
+						fprintf(stderr,"Parameters are not correct. Please try again.\n");
+						fprintf(stderr,"Public token ID must be 12 characters long.\n");
+						fprintf(stderr,"Secret token ID must be 12 characters long.\n");
+						fprintf(stderr,"AES key must be 32 characters long.\n");
+						exit(2);
+					}
+				}else if(ext_db)
+				{
+					if((strlen(user) > 16 || strlen(Argv[6])!=12 || strlen(Argv[7]) != 12 || strlen(Argv[8]) != 32))
+					{
+						fprintf(stderr,"Parameters are not correct. Please try again.\n");
+						fprintf(stderr,"Public token ID must be 12 characters long.\n");
+						fprintf(stderr,"Secret token ID must be 12 characters long.\n");
+						fprintf(stderr,"AES key must be 32 characters long.\n");
+						exit(2);
+					}
 				}
 				query = (char *)malloc((size_t)200);
-				sprintf(query,"SELECT * FROM %s WHERE nickname='%s' OR publicname='%s'",table,user,Argv[4]);
+				if(!ext_db)
+				{
+					sprintf(query,"SELECT * FROM %s WHERE nickname='%s' OR publicname='%s'",table,user,Argv[4]);
+				}else if(ext_db)
+				{
+					sprintf(query,"SELECT * FROM %s WHERE nickname='%s' OR publicname='%s'",table,user,Argv[6]);
+				}
 				retval = sqlite3_open(sqlite3_dbpath,&handle);
 				if(retval)
 				{
@@ -303,11 +326,20 @@ void db_ops(char *table,char *user,int operation,int id)
 					fprintf(stdout,"Username/key not found.Adding it into database.\n");
 					t = time(NULL);
 					tmp = localtime(&t);
-					cdatetime = (char *)malloc((char)21);
-					strftime(cdatetime,21,"%Y-%m-%dT%H:%M:%SZ",tmp);
-					sprintf(query,"INSERT INTO %s VALUES('%s','%s','%s','%s','%s',1,1,1)",table,user,Argv[4],cdatetime,Argv[5],Argv[6]);
+					ftime(&tp);
+					cdatetime = (char *)malloc((char)25);
+					cdatetime_tmp = (char *)malloc((char)20);
+					strftime(cdatetime_tmp,20,"%Y-%m-%dT%H:%M:%S",tmp);
+					sprintf(cdatetime,"%s.%.3dZ",cdatetime_tmp,tp.millitm);
+					if(!ext_db)
+					{
+						sprintf(query,"INSERT INTO %s VALUES('%s','%s','%s','%s','%s',1,1,1)",table,user,Argv[4],cdatetime,Argv[5],Argv[6]);
+					}else if(ext_db)
+					{
+						sprintf(query,"INSERT INTO %s VALUES('%s','%s','%s','%s','%s',1,1,1)",table,user,Argv[6],cdatetime,Argv[7],Argv[8]);
+					}
 					retval = sqlite3_exec(handle,query,0,0,0);
-					free(cdatetime);
+					free(cdatetime); free(cdatetime_tmp);
 					if(retval)
 					{
 						fprintf(stderr,"Adding user '%s' failed",user);
@@ -317,17 +349,36 @@ void db_ops(char *table,char *user,int operation,int id)
 					goto out;
 				}
 
-			}else if(id==OAUTH)
+			}else if(id==OATH)
 			{
-				if(strlen(user) > 16 || strlen(Argv[4])!=12 || strlen(Argv[5]) != 40)
+				if(!ext_db)
 				{
-					fprintf(stderr,"Parameters are not correct. Please try again.\n");
-					fprintf(stderr,"Public token ID must be 12 characters long.\n");
-					fprintf(stderr,"Secret key must be 40 characters long.\n");
-					exit(2);
+					if((strlen(user) > 16 || strlen(Argv[4])!=12 || strlen(Argv[5]) != 40))
+					{
+						fprintf(stderr,"Parameters are not correct. Please try again.\n");
+						fprintf(stderr,"Public token ID must be 12 characters long.\n");
+						fprintf(stderr,"Secret key must be 40 characters long.\n");
+						exit(2);
+					}
+				}
+				else  if(ext_db)
+				{
+					if((strlen(user) > 16 || strlen(Argv[6])!=12 || strlen(Argv[7]) != 40))
+					{
+						fprintf(stderr,"Parameters are not correct. Please try again.\n");
+						fprintf(stderr,"Public token ID must be 12 characters long.\n");
+						fprintf(stderr,"Secret key must be 40 characters long.\n");
+						exit(2);
+					}
+				}
+				query = (char *)calloc(1,(size_t)150);
+				if(!ext_db)
+				{
+					sprintf(query,"SELECT * FROM %s WHERE nickname='%s' OR publicname='%s'",table,user,Argv[4]);
+				}else if(ext_db)
+				{
+					sprintf(query,"SELECT * FROM %s WHERE nickname='%s' OR publicname='%s'",table,user,Argv[6]);
 				}
-				query = (char *)malloc((size_t)200);
-				sprintf(query,"SELECT * FROM %s WHERE nickname='%s' OR publicname='%s'",table,user,Argv[4]);
 				retval = sqlite3_open(sqlite3_dbpath,&handle);
 				if(retval)
 				{
@@ -355,7 +406,13 @@ void db_ops(char *table,char *user,int operation,int id)
 					tmp = localtime(&t);
 					cdatetime = (char *)malloc((char)21);
 					strftime(cdatetime,21,"%Y-%m-%dT%H:%M:%SZ",tmp);
-					sprintf(query,"INSERT INTO %s VALUES('%s','%s','%s','%s',1,1)",table,user,Argv[4],cdatetime,Argv[5]);
+					if(!ext_db)
+					{
+						sprintf(query,"INSERT INTO %s VALUES('%s','%s','%s','%s',1,1)",table,user,Argv[4],cdatetime,Argv[5]);
+					}else if(ext_db)
+					{
+						sprintf(query,"INSERT INTO %s VALUES('%s','%s','%s','%s',1,1)",table,user,Argv[6],cdatetime,Argv[7]);
+					}	
 					retval = sqlite3_exec(handle,query,0,0,0);
 					free(cdatetime);
 					if(retval)
@@ -468,27 +525,53 @@ void db_ops(char *table,char *user,int operation,int id)
 					fprintf(stderr,"Database error encountered");
 	                                exit(0);
 				}
-				if(Argc!=5)
+				if(!ext_db)
 				{
-					fprintf(stderr,"No API Key given.\n");
-					goto out;
-				}
-
-				if(strlen(Argv[4])>20 || strlen(Argv[4])<20)
+					if(Argc!=5)
+					{
+						fprintf(stderr,"No API Key given.\n");
+						goto out;
+					}
+				}else if(ext_db)
 				{
-					fprintf(stderr,"Invalid API Key. It must be 20 characters long.\n");
-					goto out;
+					if(Argc!=7)
+					{
+						fprintf(stderr,"No API Key given.\n");
+						goto out;
+					}
+				}
+				if(!ext_db)
+				{
+					if(strlen(Argv[4])>20 || strlen(Argv[4])<20)
+					{
+						fprintf(stderr,"Invalid API Key. It must be 20 characters long.\n");
+						goto out;
+					}
+					sprintf(query,"INSERT INTO apikeys VALUES ('%s','%s','%d')",user,Argv[4],api_id);
+				}
+				else if(ext_db)
+				{
+					if(strlen(Argv[6])>20 || strlen(Argv[6])<20)
+					{
+						fprintf(stderr,"Invalid API Key. It must be 20 characters long.\n");
+						goto out;
+					}
+					sprintf(query,"INSERT INTO apikeys VALUES ('%s','%s','%d')",user,Argv[6],api_id);
 				}
-				sprintf(query,"INSERT INTO apikeys VALUES ('%s','%s','%d')",user,Argv[4],api_id);
 				retval = sqlite3_exec(handle,query,0,0,0);
 				if(retval)
 				{
 					fprintf(stderr,"Adding new API KEY for '%s' failed",user);
 					goto out;
 				}
-
-				fprintf(stdout,"New API Key for '%s': %s\n",user,base64_encode((unsigned char *)Argv[4],20,20));
-				fprintf(stdout,"Your API Key ID is: %d\n",api_id);
+				if(!ext_db)
+				{
+					fprintf(stdout,"New API Key for '%s': %s\n",user,base64_encode((unsigned char *)Argv[4],20,20));
+				}else if(ext_db)
+				{
+					fprintf(stdout,"New API Key for '%s': %s\n",user,base64_encode((unsigned char *)Argv[6],20,20));
+				}
+					fprintf(stdout,"Your API Key ID is: %d\n",api_id);
 				goto out;
 			}
 			else
@@ -507,11 +590,11 @@ out:
 
 void show_table()
 {
-	if(yubikey_table && !oauth_table && !api_table)
+	if(yubikey_table && !oath_table && !api_table)
 		print_table("yubikeys",YUBIKEY);
-	else if(!yubikey_table && oauth_table && !api_table)
-		print_table("oathtokens",OAUTH);
-	else if(!yubikey_table && !oauth_table && api_table)
+	else if(!yubikey_table && oath_table && !api_table)
+		print_table("oathtokens",OATH);
+	else if(!yubikey_table && !oath_table && api_table)
 		print_table("apikeys",API);
 	else 
 		printf("Please choose a table to list.\n");
@@ -519,11 +602,11 @@ void show_table()
 
 void user_ops(char *user,int operation)
 {
-	if(yubikey_table && !oauth_table && !api_table)
+	if(yubikey_table && !oath_table && !api_table)
 		db_ops("yubikeys",user,operation,YUBIKEY);
-	else if(!yubikey_table && oauth_table && !api_table)
-		db_ops("oathtokens",user,operation,OAUTH);
-	else if(!yubikey_table && !oauth_table && api_table)
+	else if(!yubikey_table && oath_table && !api_table)
+		db_ops("oathtokens",user,operation,OATH);
+	else if(!yubikey_table && !oath_table && api_table)
 		db_ops("apikeys",user,operation,API);
 	else
 		printf("Please choose a table first.\n");
@@ -541,21 +624,21 @@ void usage()
 		       "   --help or -h				Print this help screen\n"
 		       "   --database or -b			Use this SQLite3 database file (optional)\n"
 		       "   --yubikey or -y			Choose Yubikey Token table\n"
-		       "   --oauth or -o				Choose OAUTH Token table\n"
+		       "   --oath or -o				Choose OATH Token table\n"
 		       "   --api or -p				Choose API Key table\n"
-		       "   --add N [P S [A]] or -a N [P S [A]]	Add Yubikey OTP & HOTP/OAUTH token or API Key 'N' user\n"
+		       "   --add N [P S [A]] or -a N [P S [A]]	Add Yubikey OTP & HOTP/OATH token or API Key 'N' user\n"
 		       "					where N is the username, P the Public Token ID,\n"
 		       "					S the Secret ID and A the AES Key\n"
 		       "					N must be 16 characters max,P must be 12 characters\n"
-		       "					for Yubikey OTP and 12 characters max for HOTP/OAUTH\n"
-		       "					S must be 12 characters for Yubikey OTP and 40 for HOTP/OAUTH\n"
+		       "					for Yubikey OTP and 12 characters max for HOTP/OATH\n"
+		       "					S must be 12 characters for Yubikey OTP and 40 for HOTP/OATH\n"
 		       "					and AES key must be 32 characters\n"
 		       "					Adding a user to API keys requires a username\n"
 		       "					and a API Key 20 characters long\n"
-		       "   --delete N or -x N			Delete Yubikey OTP, HOTP/OAUTH token or API Key 'N' user\n"
-		       "   --enable N or -e N			Enable Yubikey OTP, HOTP/OAUTH token 'N' user\n"
-		       "   --disable N or -d N			Disable Yubikey OTP, HOTP/OAUTH token 'N' user\n"
-		       "   --list or -l				List Yubikey OTP, HOTP/OAUTH token or API Key\n"
+		       "   --delete N or -x N			Delete Yubikey OTP, HOTP/OATH token or API Key 'N' user\n"
+		       "   --enable N or -e N			Enable Yubikey OTP, HOTP/OATH token 'N' user\n"
+		       "   --disable N or -d N			Disable Yubikey OTP, HOTP/OATH token 'N' user\n"
+		       "   --list or -l				List Yubikey OTP, HOTP/OATH token or API Key\n"
 		       "This version of yubiserver-admin has been configured with '" SQLITE3_DB_PATH "' as its default\n"
                        "SQLite3 database file.\n");
 }
@@ -569,7 +652,7 @@ int main(int argc, char **argv)
 		{"help",0,0,'h'},
 		{"list",0,0,'l'},
 		{"yubikey",0,0,'y'},
-		{"oauth",0,0,'o'},
+		{"oath",0,0,'o'},
 		{"api",0,0,'p'},
 
 		/* Takes parameters */
@@ -603,6 +686,7 @@ int main(int argc, char **argv)
 				exit(0);
 			case 'b':
 				sqlite3_dbpath = strdup(optarg);
+				ext_db = 1;
 				break;
 			case 'a':
 				Argc = argc;
@@ -614,7 +698,7 @@ int main(int argc, char **argv)
 				yubikey_table = 1;
 				break;
 			case 'o':
-				oauth_table = 1;
+				oath_table = 1;
 				break;
 			case 'p':
 				api_table = 1;
