@@ -34,6 +34,8 @@
 #include <getopt.h>
 #include <math.h>
 #include <mhash.h>
+#include <sys/types.h>
+#include <pwd.h>
 
 #include <ev.h>
 
@@ -99,7 +101,7 @@ void modhex2hex(char *otp_n,char *otp)
 	}
 	return;
 out:
-	memcpy(otp_n,otp,strlen(otp));
+	memcpy(otp_n,otp,OTP_TOKEN);
 	return;
 }
 
@@ -143,7 +145,8 @@ void htoa(char *token,char *out)
       	 	sprintf((out+2*index),"%c",hex[((unsigned char)token[index]/16)]);
 	        sprintf((out+2*index+1),"%c",hex[((unsigned char)token[index]%16)]);
         }
-        out[33]='\0';
+	BT_(out,32);
+//        out[33]='\0';
 }
 	
 /* Get Public ID from Encoded or Decoded OTP */
@@ -216,11 +219,11 @@ char *aes128ecb_decrypt(char *otp,char *premod_otp,char *private_id)
 	size_t           index;
 	size_t		 otpLength=16;
     	size_t keyLength = gcry_cipher_get_algo_keylen(GCRY_CIPHER);
-	char * decoded_otp = (char *)malloc((size_t)16);
-	char * otp_token = (char *)malloc((size_t)16);
-	char * otp_buffer = (char *)malloc((size_t)32);
-	char * final_otp = (char *)malloc(33);
-	char * aesSymKey = (char *)malloc(32);
+	char * decoded_otp = (char *)malloc((size_t)HEX_SIZE+1);
+	char * otp_token = (char *)malloc((size_t)HEX_SIZE+1);
+	char * otp_buffer = (char *)malloc((size_t)OTP_MSG_SIZE+1);
+	char * final_otp = (char *)malloc(OTP_MSG_SIZE+1);
+	char * aesSymKey = (char *)malloc(AES_SIZE+1);
 	gcry_control (GCRYCTL_ANY_INITIALIZATION_P);
 	//gcry_check_version ( NULL );
         gcry_control ( GCRYCTL_DISABLE_SECMEM_WARN );
@@ -236,6 +239,9 @@ char *aes128ecb_decrypt(char *otp,char *premod_otp,char *private_id)
 		free(aesSymKey);
 		return NULL;
 	}
+	
+	BT_(aesSymKey,AES_SIZE);
+	BT_(private_id,PRIVATE_ID_SIZE);
 
 	char * aesKey = (char *)malloc((size_t)16);
 
@@ -258,6 +264,8 @@ char *aes128ecb_decrypt(char *otp,char *premod_otp,char *private_id)
 		return NULL;
 	}
 	atoh(aesSymKey,aesKey);
+	BT_(aesKey,HEX_SIZE);
+
 	gcryError = gcry_cipher_setkey(gcryCipherHd, aesKey, keyLength);
 
 	if(gcryError)
@@ -274,19 +282,27 @@ char *aes128ecb_decrypt(char *otp,char *premod_otp,char *private_id)
 		return NULL;
 	}
 
-	memcpy(otp_buffer,otp+12,32);
+	//memcpy(otp_buffer,otp+12,32);
+	memcpy(otp_buffer,otp+12,OTP_MSG_SIZE);
+	BT_(otp_buffer,OTP_MSG_SIZE);
+
 /*	for(index=0;index<32;index++)
 	{
 		otp_buffer[index]=otp[index+12];
 	}
 */
 	atoh(otp_buffer,otp_token);
+	BT_(otp_token,HEX_SIZE);
+
 	gcryError = gcry_cipher_decrypt(
 			gcryCipherHd, // gcry_cipher_hd_t
 			decoded_otp,  // void *
 			otpLength,    // size_t
 			otp_token,    // const void *
 			keyLength);   // size_t
+
+	BT_(decoded_otp,HEX_SIZE);
+
 	if(gcryError)
 	{
 		//yubilog(ERROR,"gcry_cipher_decrypt failed",gcry_strsource(gcryError),gcry_strerror(gcryError));
@@ -367,7 +383,7 @@ char *base64_encode(const unsigned char *data,size_t input_length,size_t output_
 
 int calc_counter(char *otp)
 {
-	static char chrotpcounter[7];
+	static char chrotpcounter[OTP_TOKEN];
         sprintf(chrotpcounter,"%2s",(otp+14));
         sprintf((chrotpcounter+2),"%2s",(otp+12));
         sprintf((chrotpcounter+4),"%2s",(otp+22));
@@ -377,7 +393,7 @@ int calc_counter(char *otp)
 
 int calc_timestamp(char *otp)
 {
-	static char chrotptimestamp[7];
+	static char chrotptimestamp[OTP_TOKEN];
         sprintf(chrotptimestamp,"%2s",(otp+20));
         sprintf((chrotptimestamp+2),"%2s",(otp+18));
         sprintf((chrotptimestamp+4),"%2s",(otp+16));
@@ -387,7 +403,7 @@ int calc_timestamp(char *otp)
 
 int calc_sessioncounter(char *otp)
 {
-	static char chrotpcounter[5];
+	static char chrotpcounter[OTP_TOKEN];
         sprintf(chrotpcounter,"%2s",(otp+14));
         sprintf((chrotpcounter+2),"%2s",(otp+12));
 	chrotpcounter[4]='\0';
@@ -396,7 +412,7 @@ int calc_sessioncounter(char *otp)
 
 int calc_sessiontokencounter(char *otp)
 {
-	static char chrotpcounter[3];
+	static char chrotpcounter[OTP_TOKEN];
         sprintf(chrotpcounter,"%2s",(otp+22));
 	chrotpcounter[2]='\0';
 	return (int)strtol(chrotpcounter,NULL,16);
@@ -681,7 +697,7 @@ struct Yubikey *oath_counter_secret(char *id)
 		if(retval==SQLITE_ROW)
 		{
 			oyubikey->counter = atoi((const char *)sqlite3_column_text(stmt,0));
-			sprintf(oyubikey->private_id,"%s",sqlite3_column_text(stmt,1));
+			sprintf(oyubikey->oprivate_id,"%s",sqlite3_column_text(stmt,1));
 			rows++;
 		}
 		else if(retval==SQLITE_DONE)
@@ -765,7 +781,7 @@ int validate_hotp(char *id,char *otp,struct OATH_Tokens *tokens)
 		return BAD_OTP;
 	}
 	Key = (char *)malloc((size_t)21);
-	oath_atoh(oyubikey->private_id,Key);
+	oath_atoh(oyubikey->oprivate_id,Key);
 	Key[20]='\0';
 	for(counter=1+oyubikey->counter ; counter < 256+oyubikey->counter; counter++)
 	{
@@ -883,7 +899,7 @@ static void write_callback(struct ev_loop *loop, struct ev_io *w, int revents)
 	char *private_id = NULL;
 	struct Tokens  *tokens = NULL;
 	struct OATH_Tokens  *oath_tokens = NULL;
-	static char validation_date[24];
+	static char validation_date[25];
 	static char datetmp[20];
 	time_t t;
 	struct tm *tmp = NULL;
@@ -937,7 +953,9 @@ static void write_callback(struct ev_loop *loop, struct ev_io *w, int revents)
 			if(tokens->otp != NULL)
 			{
 				otp=(char *)malloc((size_t)OTP_TOKEN);
+				if(strlen(tokens->otp) > OTP_TOKEN) BT_(tokens->otp,OTP_TOKEN);
 				modhex2hex(otp,tokens->otp);
+				BT_(otp,OTP_TOKEN);
 				otp_n = aes128ecb_decrypt(otp,tokens->otp,private_id); /* otp is now decrypted if it is valid and if user exists */
 				if(otp != NULL) free(otp);
 			}
@@ -955,6 +973,7 @@ static void write_callback(struct ev_loop *loop, struct ev_io *w, int revents)
 			tmp = localtime(&t);
 			strftime(datetmp,20,"%FT%T",tmp);
 			sprintf(validation_date,"%s.%.3dZ",datetmp,tp.millitm);
+			BT_(validation_date,24);
 			if(tokens->nonce != NULL)
 				if(strlen(tokens->nonce) < 16 || strlen(tokens->nonce) > 40)
 					tokens->nonce = NULL;
@@ -1013,6 +1032,7 @@ static void write_callback(struct ev_loop *loop, struct ev_io *w, int revents)
 			tmp = localtime(&t);
 			strftime(datetmp,20,"%FT%T",tmp);
 			sprintf(validation_date,"%s.%.3dZ",datetmp,tp.millitm);
+			BT_(validation_date,24);
 			if(oath_tokens->otp != NULL && ( strlen(oath_tokens->otp) == 18 || strlen(oath_tokens->otp) == 20  ) )
 				result = validate_all(oath_tokens->otp,NULL,NULL,NULL,NULL,METHOD_OATH,&oath_tokens->otp);
 			else
@@ -1028,7 +1048,7 @@ static void write_callback(struct ev_loop *loop, struct ev_io *w, int revents)
 			free(oath_tokens);
 			ev_io_stop(EV_A_ w);
 		}
-	}else if(cli->mode=EV_DEFAULT_PAGE) /* Default HTTP/1.0 request */
+	}else if(cli->mode==EV_DEFAULT_PAGE) /* Default HTTP/1.0 request */
 	{
 		if (revents & EV_WRITE)
 		{
@@ -1216,6 +1236,10 @@ int main(int argc, char **argv)
 		(void)close(i);		/* close open files */
 	
 	(void)setpgrp();		/* break away from process group */
+	/* drop privileges , change to yubiserver user */
+	struct passwd *yubiserver_user = getpwnam("yubiserver");
+	(void)setreuid(yubiserver_user->pw_uid,yubiserver_user->pw_uid);
+	(void)setregid(yubiserver_user->pw_gid,yubiserver_user->pw_gid);
 
 	/* setup the network socket */
 	if((listenfd = socket(AF_INET, SOCK_STREAM,0)) <0)
