@@ -7,7 +7,7 @@
 #define ERROR           42
 #define WARNING         43
 #define LOG             44
-#define VERSION_       	"0.2"
+#define VERSION_       	"0.3"
 #define METHOD_OTP      1
 #define METHOD_OATH     2
 
@@ -25,12 +25,16 @@
 
 #define NO_AUTH			11	/* The OATH/HOTP is invalid. */
 
-#define CRC_OK          0xF0B8
-#define CRC_BLOCK_SIZE  16
-#define PRIVATE_ID_SIZE 12
-#define OTP_MSG_SIZE    32
-#define OTP_TOKEN       44
-#define QUERY_SIZE      100
+#define CRC_OK          	0xF0B8
+#define CRC_BLOCK_SIZE  	16
+#define PRIVATE_ID_SIZE 	12
+#define OPRIVATE_ID_SIZE 	40
+#define OTP_MSG_SIZE    	32
+#define OTP_TOKEN       	44
+#define AES_SIZE		32
+#define HEX_SIZE		16
+#define PUBLIC_NAME_SIZE	16
+#define QUERY_SIZE      	100
 
 /* Yubiserver-admin Constants */
 #define ENABLE_USER	0
@@ -51,6 +55,8 @@
         const typeof( ((type *)0)->member ) *__mptr = (ptr);    \
         (type *)( (char *)__mptr - offsetof(type,member) );})
 
+#define BT_(x,y)	(x[y]='\0')
+
 /* Change default path to /etc/yubiserver/yubiserver.sqlite */
 //#define SQLITE3_DB_PATH		"yubiserver.sqlite"
 //#define YUBISERVER_LOG_PATH	"yubiserver.log"
@@ -59,16 +65,17 @@ char *sqlite3_dbpath = SQLITE3_DB_PATH;
 char *yubiserver_log = YUBISERVER_LOG_PATH;
 
 struct Yubikey {
-	int result;		/* Final Result after validation */
-	char publicname[16];	/* Database Public Name */
-	char creation_date[25];	/* Database account creation datetime  */
-	char private_id[12];	/* Database private ID */
-	char aeskey[32];	/* Database AES Key */
-	int active;		/* Account is active */
-	int counter;		/* Database counter */
-	int timestamp;		/* Database timestamp */
-	int session_counter;	/* Internal session counter */
-	int session_token_counter;	/* Internal session token counter */
+	int result;				/* Final Result after validation */
+	char publicname[PUBLIC_NAME_SIZE+1];	/* Database Public Name */
+	char creation_date[25];			/* Database account creation datetime  */
+	char private_id[PRIVATE_ID_SIZE+1];	/* Database private ID */
+	char oprivate_id[OPRIVATE_ID_SIZE+1];	/* Database OATH private ID */
+	char aeskey[AES_SIZE+1];		/* Database AES Key */
+	int active;				/* Account is active */
+	int counter;				/* Database counter */
+	int timestamp;				/* Database timestamp */
+	int session_counter;			/* Internal session counter */
+	int session_token_counter;		/* Internal session token counter */
 };
 
 struct Tokens {
